==> dunelegacy-0.96.2-src.tar.bz2: dunelegacy-0.96.2/src/structures/Refinery.cpp <==
/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <structures/Refinery.h>

#include <globals.h>

#include <FileClasses/GFXManager.h>
#include <House.h>
#include <SoundPlayer.h>
#include <Map.h>

#include <units/UnitBase.h>
#include <units/Harvester.h>
#include <units/Carryall.h>

#include <GUI/ObjectInterfaces/RefineryAndSiloInterface.h>

Refinery::Refinery(House* newOwner) : StructureBase(newOwner)
{
    Refinery::init();

    setHealth(getMaxHealth());

	extractingSpice = false;
    bookings = 0;

	firstRun = true;

    FirstAnimFrame = 2;
	LastAnimFrame = 3;
}

Refinery::Refinery(InputStream& stream) : StructureBase(stream)
{
    Refinery::init();

	extractingSpice = stream.readBool();
	harvester.load(stream);
	bookings = stream.readUint32();

	if(extractingSpice) {
		FirstAnimFrame = 8;
		LastAnimFrame = 9;
		curAnimFrame = 8;
	} else if(bookings == 0) {
		stopAnimate();
	} else {
		startAnimate();
	}

	firstRun = false;
}
void Refinery::init()
{
    itemID = Structure_Refinery;
	owner->incrementStructures(itemID);

	structureSize.x = 3;
	structureSize.y = 2;

	GraphicID = ObjPic_Refinery;
	graphic = pGFXManager->getObjPic(GraphicID,getOwner()->getHouseID());
	numImagesX = 10;
	numImagesY = 1;
}

Refinery::~Refinery()
{
    if (extractingSpice && harvester) {
		if(harvester.getUnitPointer() != NULL)
			harvester.getUnitPointer()->destroy();
		harvester.PointTo(NONE);
	}
}

void Refinery::save(OutputStream& stream) const
{
	StructureBase::save(stream);

    stream.writeBool(extractingSpice);
	harvester.save(stream);
	stream.writeUint32(bookings);
}

ObjectInterface* Refinery::GetInterfaceContainer() {
	if((pLocalHouse == owner) || (debug == true)) {
		return RefineryAndSiloInterface::Create(objectID);
	} else {
		return DefaultObjectInterface::Create(objectID);
	}
}

void Refinery::assignHarvester(Harvester* newHarvester)
{
	extractingSpice = true;
	harvester.PointTo(newHarvester);
	drawnAngle = 1;
	FirstAnimFrame = 8;
	LastAnimFrame = 9;
	curAnimFrame = 8;
}



void Refinery::deployHarvester(Carryall* pCarryall)
{
	unBook();
	drawnAngle = 0;
	extractingSpice = false;

	if(firstRun) {
		if (getOwner() == pLocalHouse)
		{
			soundPlayer->playVoice(HarvesterDeployed,getOwner()->getHouseID());
		}
	}

	firstRun = false;

    Harvester* pHarvester = (Harvester*) harvester.getObjPointer();
	if(pCarryall != NULL) {
	    pCarryall->giveCargo(pHarvester);
	    pCarryall->setTarget(NULL);
	    pCarryall->setDestination(pHarvester->getGuardPoint());
	} else {
        Coord deployPos = currentGameMap->findDeploySpot(pHarvester, location, destination, structureSize);
        pHarvester->deploy(deployPos);
	}

	if(bookings == 0) {
        stopAnimate();
	} else {
        startAnimate();
	}
}

void Refinery::startAnimate()
{
	if(extractingSpice == false) {
		FirstAnimFrame = 2;
		LastAnimFrame = 7;
		curAnimFrame = 2;
	}
}

void Refinery::stopAnimate()
{
    FirstAnimFrame = 2;
	LastAnimFrame = 3;
	curAnimFrame = 2;
}

void Refinery::doSpecificStuff()
{
	if(extractingSpice) {
	    Harvester* pHarvester = (Harvester*) harvester.getObjPointer();

		if(pHarvester->getAmountOfSpice() > 0.0f) {
			owner->addCredits(pHarvester->extractSpice(), true);
		} else if(pHarvester->isAwaitingPickup() == false) {
		    // find carryall
		    Carryall* pCarryall = NULL;
            if((pHarvester->getGuardPoint().x != INVALID_POS) && getOwner()->hasCarryalls())	{
                RobustList<UnitBase*>::const_iterator iter;
                for(iter = unitList.begin(); iter != unitList.end(); ++iter) {
                    UnitBase* unit = *iter;
                    if ((unit->getOwner() == owner) && (unit->getItemID() == Unit_Carryall)) {
                        if (((Carryall*)unit)->isRespondable() && !((Carryall*)unit)->isBooked()) {
                            pCarryall = (Carryall*)unit;
                        }
                    }
                }
            }

            if(pCarryall != NULL) {
                pCarryall->setTarget(this);
                pCarryall->clearPath();
                pHarvester->bookCarrier(pCarryall);
                pHarvester->setTarget(NULL);
                pHarvester->setDestination(pHarvester->getGuardPoint());
            } else {
                deployHarvester();
            }
		} else if(pHarvester->getCarrier() == NULL) {
		    deployHarvester();
		}
	}
}

void Refinery::destroy()
{
	StructureBase::destroy();
}

==> dunelegacy-0.96.3-src.tar.bz2: dunelegacy-0.96.3/src/structures/Refinery.cpp <==
/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <structures/Refinery.h>

#include <globals.h>

#include <FileClasses/GFXManager.h>
#include <House.h>
#include <SoundPlayer.h>
#include <Map.h>

#include <units/UnitBase.h>
#include <units/Harvester.h>
#include <units/Carryall.h>

#include <GUI/ObjectInterfaces/RefineryAndSiloInterface.h>

/* how fast is spice extracted */
#define MAXIMUMHARVESTEREXTRACTSPEED 0.625f

Refinery::Refinery(House* newOwner) : StructureBase(newOwner) {
    Refinery::init();

    setHealth(getMaxHealth());

	extractingSpice = false;
    bookings = 0;

	firstRun = true;

    firstAnimFrame = 2;
	lastAnimFrame = 3;
}

Refinery::Refinery(InputStream& stream) : StructureBase(stream) {
    Refinery::init();

	extractingSpice = stream.readBool();
	harvester.load(stream);
	bookings = stream.readUint32();

	if(extractingSpice) {
		firstAnimFrame = 8;
		lastAnimFrame = 9;
		curAnimFrame = 8;
	} else if(bookings == 0) {
		stopAnimate();
	} else {
		startAnimate();
	}

	firstRun = false;
}

void Refinery::init() {
    itemID = Structure_Refinery;
	owner->incrementStructures(itemID);

	structureSize.x = 3;
	structureSize.y = 2;

	graphicID = ObjPic_Refinery;
	graphic = pGFXManager->getObjPic(graphicID,getOwner()->getHouseID());
	numImagesX = 10;
	numImagesY = 1;
}

Refinery::~Refinery() {
    if(extractingSpice && harvester) {
		if(harvester.getUnitPointer() != NULL)
			harvester.getUnitPointer()->destroy();
		harvester.pointTo(NONE);
	}
}

void Refinery::save(OutputStream& stream) const {
	StructureBase::save(stream);

    stream.writeBool(extractingSpice);
	harvester.save(stream);
	stream.writeUint32(bookings);
}

ObjectInterface* Refinery::getInterfaceContainer() {
	if((pLocalHouse == owner) || (debug == true)) {
		return RefineryAndSiloInterface::create(objectID);
	} else {
		return DefaultObjectInterface::create(objectID);
	}
}

void Refinery::assignHarvester(Harvester* newHarvester) {
	extractingSpice = true;
	harvester.pointTo(newHarvester);
	drawnAngle = 1;
	firstAnimFrame = 8;
	lastAnimFrame = 9;
	curAnimFrame = 8;
}

void Refinery::deployHarvester(Carryall* pCarryall) {
	unBook();
	drawnAngle = 0;
	extractingSpice = false;

	if(firstRun) {
		if(getOwner() == pLocalHouse) {
			soundPlayer->playVoice(HarvesterDeployed,getOwner()->getHouseID());
		}
	}

	firstRun = false;

    Harvester* pHarvester = (Harvester*) harvester.getObjPointer();
	if(pCarryall != NULL) {
	    pCarryall->giveCargo(pHarvester);
	    pCarryall->setTarget(NULL);
	    pCarryall->setDestination(pHarvester->getGuardPoint());
	} else {
        Coord deployPos = currentGameMap->findDeploySpot(pHarvester, location, destination, structureSize);
        pHarvester->deploy(deployPos);
	}

	if(bookings == 0) {
        stopAnimate();
	} else {
        startAnimate();
	}
}

void Refinery::startAnimate() {
	if(extractingSpice == false) {
		firstAnimFrame = 2;
		lastAnimFrame = 7;
		curAnimFrame = 2;
        justPlacedTimer = 0;
        animationCounter = 0;
	}
}

void Refinery::stopAnimate() {
    firstAnimFrame = 2;
	lastAnimFrame = 3;
	curAnimFrame = 2;
}

void Refinery::updateStructureSpecificStuff() {
	if(extractingSpice) {
	    Harvester* pHarvester = (Harvester*) harvester.getObjPointer();

		if(pHarvester->getAmountOfSpice() > 0.0f) {
		    float extractionSpeed = MAXIMUMHARVESTEREXTRACTSPEED;

		    int scale = (int) (5*getHealth()/getMaxHealth());
		    if(scale == 0) {
                scale = 1;
		    }

		    extractionSpeed = (extractionSpeed * scale) / 5;


			owner->addCredits(pHarvester->extractSpice(extractionSpeed), true);
		} else if(pHarvester->isAwaitingPickup() == false) {
		    // find carryall
		    Carryall* pCarryall = NULL;
            if((pHarvester->getGuardPoint().isValid()) && getOwner()->hasCarryalls())	{
                RobustList<UnitBase*>::const_iterator iter;
                for(iter = unitList.begin(); iter != unitList.end(); ++iter) {
                    UnitBase* unit = *iter;
                    if ((unit->getOwner() == owner) && (unit->getItemID() == Unit_Carryall)) {
                        if (((Carryall*)unit)->isRespondable() && !((Carryall*)unit)->isBooked()) {
                            pCarryall = (Carryall*)unit;
                        }
                    }
                }
            }

            if(pCarryall != NULL) {
                pCarryall->setTarget(this);
                pCarryall->clearPath();
                pHarvester->bookCarrier(pCarryall);
                pHarvester->setTarget(NULL);
                pHarvester->setDestination(pHarvester->getGuardPoint());
            } else {
                deployHarvester();
            }
		} else if(!pHarvester->hasBookedCarrier()) {
		    deployHarvester();
		}
	}
}
